--
-- $Id: Config.hs,v 1.2 2005/12/16 20:10:00 aamine Exp $
--
-- Copyright (c) 2005 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LGPL, see the file "COPYING".
--

module Config
    (Config, loadConfig, parseConfigFile, confLookup,
     confLookupString, confLookupPath, confLookupBool) where

import TextUtils
import Data.List
import Data.Maybe

type Config = [(String, String)]

confLookup :: Eq a => a -> [(a, b)] -> b
confLookup key = fromJust . lookup key

confLookupString :: String -> [(String, String)] -> String
confLookupString = confLookup

confLookupPath :: String -> [(String, String)] -> FilePath
confLookupPath = confLookupString

confLookupBool :: String -> [(String, String)] -> Bool
confLookupBool key = read . confLookup key

loadConfig :: FilePath -> IO [(String, [(String, String)])]
loadConfig path = return . parseConfigFile =<< readFile path

parseConfigFile :: String -> [(String, [(String, String)])]
parseConfigFile = map (reduce) . categorize . concatMap (parseLine) . lines
  where
    reduce :: [(String, String)] -> (String, [(String, String)])
    reduce kvs = (category . fst . head $ kvs,
                  map (\(k,v) -> (itemname k, v)) kvs)

    categorize = groupBy (\(a,_) (b,_) -> category a == category b)

    -- a key of the config item = "category.itemname"
    category = fst . break (== '.')
    itemname = tail . snd . break (== '.')

    parseLine = parseLinePlain . dropComment

    parseLinePlain line
        | isBlank line = []
        | otherwise    = let (k, ('=':v)) = break (== '=') . strip $ line
                         in [(strip k, strip v)]

    dropComment = fst . break (== '#')
